==> jboss-3.2.6-src.tar.bz2: jboss-3.2.6-src/testsuite/src/main/org/jboss/test/jbossmq/test/ConcurrentDeliveryUnitTestCase.java <==
/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.test.jbossmq.test;

import javax.jms.Message;
import javax.jms.MessageListener;
import javax.jms.QueueConnection;
import javax.jms.QueueConnectionFactory;
import javax.jms.QueueReceiver;
import javax.jms.QueueSender;
import javax.jms.QueueSession;
import javax.jms.Session;
import javax.jms.TemporaryQueue;
import javax.naming.Context;

import org.jboss.test.JBossTestCase;

/**
 * Concurrent delivery tests
 *
 * @author <a href="mailto:adrian@jboss.org>Adrian Brock</a>
 * @version <tt>$Revision: 1.1.2.1 $</tt>
 */
public class ConcurrentDeliveryUnitTestCase extends JBossTestCase
{
   static String QUEUE_FACTORY = "ConnectionFactory";

   QueueConnection queueConnection;

   int completed = 0;
   boolean inDelivery = false;
   boolean concurrent = false;
   
   public ConcurrentDeliveryUnitTestCase(String name) throws Exception
   {
      super(name);
   }

   public void testConcurrentDelivery() throws Exception
   {
      connect();
      try
      {
         MyMessageListener messageListener = new MyMessageListener(); 
         
         QueueSession session = queueConnection.createQueueSession(false, Session.AUTO_ACKNOWLEDGE);
         TemporaryQueue queue1 = session.createTemporaryQueue();
         QueueSender sender1 = session.createSender(queue1);
         QueueReceiver receiver1 = session.createReceiver(queue1);
         receiver1.setMessageListener(messageListener);
         TemporaryQueue queue2 = session.createTemporaryQueue();
         QueueSender sender2 = session.createSender(queue2);
         QueueReceiver receiver2 = session.createReceiver(queue2);
         receiver2.setMessageListener(messageListener);
         Message message = session.createMessage();
         queueConnection.start();

         sender1.send(message);
         sender2.send(message);

         synchronized (messageListener)
         {
            while (completed < 2)
            {
               getLog().debug("Waiting for completion " + completed);
               messageListener.wait();
            }
         }
         getLog().debug("Completed");
         
         if (concurrent)
            fail("Concurrent delivery");
      }
      finally
      {
         disconnect();
      }
   }

   protected void connect() throws Exception
   {
      Context context = getInitialContext();
      QueueConnectionFactory queueFactory = (QueueConnectionFactory) context.lookup(QUEUE_FACTORY);
      queueConnection = queueFactory.createQueueConnection();

      getLog().debug("Connection established.");
   }

   protected void disconnect()
   {
      try
      {
         if (queueConnection != null)
            queueConnection.close();
      }
      catch (Throwable ignored)
      {
         getLog().warn("Ignored", ignored);
      }

      getLog().debug("Connection closed.");
   }

   public class MyMessageListener implements MessageListener
   {
      public void onMessage(Message message)
      {
         synchronized (this)
         {
            if (inDelivery)
               concurrent = true;
            inDelivery = true;
            getLog().debug("In delivery " + message);
         }
         
         try
         {
            Thread.sleep(10000);
         }
         catch (Throwable ignored)
         {
            getLog().warn("Ignored ", ignored);
         }
         
         synchronized (this)
         {
            inDelivery = false;
            ++completed;
            notifyAll();
            getLog().debug("Completed " + message);
         }
      }
   }
}

==> jboss-3.2.7-src.tar.bz2: jboss-3.2.7-src/testsuite/src/main/org/jboss/test/jbossmq/test/ConcurrentDeliveryUnitTestCase.java <==
/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.test.jbossmq.test;

import javax.jms.Message;
import javax.jms.MessageListener;
import javax.jms.QueueConnection;
import javax.jms.QueueConnectionFactory;
import javax.jms.QueueReceiver;
import javax.jms.QueueSender;
import javax.jms.QueueSession;
import javax.jms.Session;
import javax.jms.TemporaryQueue;
import javax.naming.Context;

import org.apache.log4j.Logger;
import org.jboss.test.JBossTestCase;

/**
 * Concurrent delivery tests
 *
 * @author <a href="mailto:adrian@jboss.org>Adrian Brock</a>
 * @version <tt>$Revision: 1.1.2.2 $</tt>
 */
public class ConcurrentDeliveryUnitTestCase extends JBossTestCase
{
   static String QUEUE_FACTORY = "ConnectionFactory";

   QueueConnection queueConnection;

   int completed = 0;
   boolean inDelivery = false;
   boolean concurrent = false;
   
   public ConcurrentDeliveryUnitTestCase(String name) throws Exception
   {
      super(name);
   }

   public void testConcurrentDelivery() throws Exception
   {
      connect();
      try
      {
         MyMessageListener messageListener = new MyMessageListener(getLog()); 
         
         QueueSession session = queueConnection.createQueueSession(false, Session.AUTO_ACKNOWLEDGE);
         TemporaryQueue queue1 = session.createTemporaryQueue();
         QueueSender sender1 = session.createSender(queue1);
         QueueReceiver receiver1 = session.createReceiver(queue1);
         receiver1.setMessageListener(messageListener);
         TemporaryQueue queue2 = session.createTemporaryQueue();
         QueueSender sender2 = session.createSender(queue2);
         QueueReceiver receiver2 = session.createReceiver(queue2);
         receiver2.setMessageListener(messageListener);
         Message message = session.createMessage();
         queueConnection.start();

         sender1.send(message);
         sender2.send(message);

         synchronized (messageListener)
         {
            while (completed < 2)
            {
               getLog().debug("Waiting for completion " + completed);
               messageListener.wait();
            }
         }
         getLog().debug("Completed");
         
         if (concurrent)
            fail("Concurrent delivery");
      }
      finally
      {
         disconnect();
      }
   }

   protected void connect() throws Exception
   {
      Context context = getInitialContext();
      System.out.println(context.getEnvironment());
      QueueConnectionFactory queueFactory = (QueueConnectionFactory) context.lookup(QUEUE_FACTORY);
      queueConnection = queueFactory.createQueueConnection();

      getLog().debug("Connection established.");
   }

   protected void disconnect()
   {
      try
      {
         if (queueConnection != null)
            queueConnection.close();
      }
      catch (Throwable ignored)
      {
         getLog().warn("Ignored", ignored);
      }

      getLog().debug("Connection closed.");
   }

   public class MyMessageListener implements MessageListener
   {
      final Logger log;
      MyMessageListener(Logger log)
      {
         this.log = log;
      }
      public void onMessage(Message message)
      {
         synchronized (this)
         {
            if (inDelivery)
               concurrent = true;
            inDelivery = true;
            log.debug("In delivery " + message);
         }
         
         try
         {
            Thread.sleep(10000);
         }
         catch (Throwable ignored)
         {
            log.warn("Ignored ", ignored);
         }
         
         synchronized (this)
         {
            inDelivery = false;
            ++completed;
            notifyAll();
            log.debug("Completed " + message);
         }
      }
   }
}
